@@ -254,8 +254,8 @@ resp_spct <-
     if (use.hinges) {
       all.hinges <- NULL
       for (wb in w.band) {
-        if (!is.null(wb$hinges) && length(wb$hinges) > 0) {
-          all.hinges <- c(all.hinges, wb$hinges)
+        if (!is.null(wb[["hinges"]]) && length(wb[["hinges"]]) > 0) {
+          all.hinges <- c(all.hinges, wb[["hinges"]])
         }
       }
       if (!is.null(all.hinges)) {
